"""Autogenerated API"""
from argus_api import session
import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module
from argus_api.utils import deprecated_alias
log = logging.getLogger(__name__)


@register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module
)
def add_openid_provider(
    shortName: str = None,
    name: str = None,
    clientID: str = None,
    customer: str = None,
    domain: str = None,
    jwks: str = None,
    issuer: str = None,
    certificate: str = None,
    providerURI: str = None,
    claimsMapping: dict = None,
    responseType: str = None,
    authorizationPrompt: str = None,
    scope: str = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Define new OpenID provider (INTERNAL)
    
    :param str shortName: Set a shortname for this provider  \=\> \[a\-zA\-Z0\-9\_\\\-\\.\]\*
    :param str name: The name of this provider  \=\> \[\\s\\w\\\{\\\}\\\$\\\-\\\(\\\)\\.\\\[\\\]\"\\\'\_\/\\\\\,\\\*\\\+\\\#\:\@\!\?\;\=\]\*
    :param str clientID: The OAuth ClientID of this provider 
    :param str customer: The id or shortname of the customer this provider is bound to. If customer is a customer group\, subcustomer users are also eligible for this provider. If not set\, this provider will be domain global. 
    :param str domain: The id or shortname of the domain this provider is bound to\, if this is a domain\-global provider. If not set\, customer must be set. 
    :param str jwks: Base64\-encoding of a valid JWKS file which can be used to verify OpenID tokens. Either jwks or certificate is required. If certificate is set\, it will be used. 
    :param str issuer: Define a specific issuer to require on id\-tokens. If not set\, the issuer is expected to be a prefix of the providerURI. 
    :param str certificate: A valid X509 certificate which can be used to verify OpenID tokens. Either jwks or certificate is required. Will override jwks if set. 
    :param str providerURI: The URI to redirect to to initiate OpenID authentication flow  \=\> \(\(https\?\|ftp\|gopher\|telnet\|file\)\:\(\(\/\)\|\(\\\\\)\)\+\[\\w\\d\:\\\#\@\%\/\;\$\(\)\~\_\?\\\\\+\-\=\\\\\\.\&\]\*\)
    :param dict claimsMapping: 
    :param str responseType: Define response type to request from provider \(default idToken\)
    :param str authorizationPrompt: Define prompt type to request on authorization \(default login\)
    :param list scope: Define extra scopes to request \(in addition to openid\)  \=\> Sanitize by regex \[\\x21\\x5D\-\\x7E\\x23\-\\x5B\]\+
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises InvalidArgumentException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/authentication/v1/openid/provider".format()

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}
    # Only send shortName if the argument was provided, dont send null values
    if shortName is not None:
        body.update({"shortName": shortName})
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({"name": name})
    # Only send clientID if the argument was provided, dont send null values
    if clientID is not None:
        body.update({"clientID": clientID})
    # Only send customer if the argument was provided, dont send null values
    if customer is not None:
        body.update({"customer": customer})
    # Only send domain if the argument was provided, dont send null values
    if domain is not None:
        body.update({"domain": domain})
    # Only send jwks if the argument was provided, dont send null values
    if jwks is not None:
        body.update({"jwks": jwks})
    # Only send issuer if the argument was provided, dont send null values
    if issuer is not None:
        body.update({"issuer": issuer})
    # Only send certificate if the argument was provided, dont send null values
    if certificate is not None:
        body.update({"certificate": certificate})
    # Only send providerURI if the argument was provided, dont send null values
    if providerURI is not None:
        body.update({"providerURI": providerURI})
    # Only send claimsMapping if the argument was provided, dont send null values
    if claimsMapping is not None:
        body.update({"claimsMapping": claimsMapping})
    # Only send responseType if the argument was provided, dont send null values
    if responseType is not None:
        body.update({"responseType": responseType})
    # Only send authorizationPrompt if the argument was provided, dont send null values
    if authorizationPrompt is not None:
        body.update({"authorizationPrompt": authorizationPrompt})
    # Only send scope if the argument was provided, dont send null values
    if scope is not None:
        body.update({"scope": scope})

    query_parameters = {}
    log.debug("POST %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.post(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

@register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module
)
def delete_openid_provider(
    idOrShortname: str,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Delete OpenID provider (INTERNAL)
    
    :param str idOrShortname: ID or shortname of provider to delete
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/authentication/v1/openid/provider/{idOrShortname}".format(idOrShortname=idOrShortname)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}

    query_parameters = {}
    log.debug("DELETE %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.delete(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

@register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module
)
def get_openid_provider(
    idOrShortname: str,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Fetch OpenID provider (INTERNAL)
    
    :param str idOrShortname: ID or shortname of provider to fetch
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/authentication/v1/openid/provider/{idOrShortname}".format(idOrShortname=idOrShortname)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}

    query_parameters = {}
    log.debug("GET %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

@register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module
)
def list_openid_providers(
    customer: str = None,
    keywords: str = None,
    sortBy: str = None,
    limit: int = 25,
    offset: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """List OpenID providers (INTERNAL)
    
    :param list customer: Limit results to providers for this customer \(or subcustomers\)
    :param list keywords: Limit results to providers matching these keywords
    :param list sortBy: Specify sort order
    :param int limit: Limit number of results
    :param int offset: Skip a number of results
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises InvalidArgumentsException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/authentication/v1/openid/provider".format(limit=limit,
        customer=customer,
        keywords=keywords,
        sortBy=sortBy,
        offset=offset)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}

    query_parameters = {}
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        query_parameters.update({"limit": limit})
    
    # Only send customer if the argument was provided, dont send null values
    if customer is not None:
        query_parameters.update({"customer": customer})
    
    # Only send keywords if the argument was provided, dont send null values
    if keywords is not None:
        query_parameters.update({"keywords": keywords})
    
    # Only send sortBy if the argument was provided, dont send null values
    if sortBy is not None:
        query_parameters.update({"sortBy": sortBy})
    
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        query_parameters.update({"offset": offset})
    
    log.debug("GET %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

@register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module
)
def openid_authentication(
    provider: str,
    idToken: str = None,
    requestedAuthorizations: str = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Submit OpenID token for validation to create a new user session (PUBLIC)
    
    :param str provider: ID or shortname of OpenID provider
    :param str idToken: ID token from provider 
    :param list requestedAuthorizations: Allow client to request authorizations as part of the authentication transaction. The client is not guaranteed to get the requested authorizations. The setPassword authorization is only returned if the current user has the FORCE\_PW\_CHANGE flag set. 
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/authentication/v1/openid/provider/{provider}/authentication".format(provider=provider)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}
    # Only send idToken if the argument was provided, dont send null values
    if idToken is not None:
        body.update({"idToken": idToken})
    # Only send requestedAuthorizations if the argument was provided, dont send null values
    if requestedAuthorizations is not None:
        body.update({"requestedAuthorizations": requestedAuthorizations})

    query_parameters = {}
    log.debug("POST %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.post(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

@register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module
)
def openid_pre_authentication(
    provider: str,
    state: str = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Request redirect URI to OpenID provider (PUBLIC)
    
    :param str provider: ID or shortname of OpenID provider
    :param str state: State to retain
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises InvalidArgumentException: on 412
    :raises TooManyPendingRequestsException: on 429
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/authentication/v1/openid/provider/{provider}/authentication".format(provider=provider,
        state=state)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}

    query_parameters = {}
    # Only send state if the argument was provided, dont send null values
    if state is not None:
        query_parameters.update({"state": state})
    
    log.debug("GET %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

@register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module
)
def request_openid_authorization_redirect(
    operation: str = None,
    context: dict = None,
    nextURI: str = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Register a user authorization request, and request a redirect to the OpenID Identity Provider to validate user presence. (INTERNAL)
    
    :param str operation: The name of the operation to authorize 
    :param dict context: Context variables to scope this authorization. All context variables required by the executing service must be present and equal to those provided here. 
    :param str nextURI: The URI to redirect\/route to after successful authorization. The URI will be validated according to policy. The authorization token returned from successful authorization should be appended as a query parameter to this URI. 
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises NotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/authentication/v1/openid/authorize/redirect".format()

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}
    # Only send operation if the argument was provided, dont send null values
    if operation is not None:
        body.update({"operation": operation})
    # Only send context if the argument was provided, dont send null values
    if context is not None:
        body.update({"context": context})
    # Only send nextURI if the argument was provided, dont send null values
    if nextURI is not None:
        body.update({"nextURI": nextURI})

    query_parameters = {}
    log.debug("POST %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.post(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

@register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module
)
def search_openid_providers(
    customer: str = None,
    keywords: str = None,
    sortBy: str = None,
    limit: int = 25,
    offset: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Search OpenID providers (INTERNAL)
    
    :param list customer: Limit result by customer ID or shortname 
    :param list keywords: Match providers by keywords 
    :param list sortBy: Specify sort order \(default shortName\)
    :param int limit: Limit the number of results \(default 25\)
    :param int offset: Skip the first results\, in sort order \(default 0\)
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises InvalidArgumentsException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/authentication/v1/openid/provider/search".format()

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}
    # Only send customer if the argument was provided, dont send null values
    if customer is not None:
        body.update({"customer": customer})
    # Only send keywords if the argument was provided, dont send null values
    if keywords is not None:
        body.update({"keywords": keywords})
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        body.update({"limit": limit})
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        body.update({"offset": offset})
    # Only send sortBy if the argument was provided, dont send null values
    if sortBy is not None:
        body.update({"sortBy": sortBy})

    query_parameters = {}
    log.debug("POST %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.post(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

@register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module
)
def submit_openid_authorization(
    stateID: str = None,
    idToken: str = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Receive the ID token redirect response for a pending User Authorization (INTERNAL)
    
    :param str stateID: The OpenID state identifier 
    :param str idToken: The ID token returned by the OpenID Provider 
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises NotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/authentication/v1/openid/authorize".format()

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}
    # Only send stateID if the argument was provided, dont send null values
    if stateID is not None:
        body.update({"stateID": stateID})
    # Only send idToken if the argument was provided, dont send null values
    if idToken is not None:
        body.update({"idToken": idToken})

    query_parameters = {}
    log.debug("POST %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.post(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

@register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module
)
def update_openid_provider(
    idOrShortname: str,
    shortName: str = None,
    name: str = None,
    clientID: str = None,
    issuer: str = None,
    jwks: str = None,
    certificate: str = None,
    providerURI: str = None,
    responseType: str = None,
    authorizationPrompt: str = None,
    claimsMapping: dict = None,
    scope: str = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Update OpenID provider (INTERNAL)
    
    :param str idOrShortname: ID or shortname of provider to update
    :param str shortName: If set\, update the shortname for this provider  \=\> \[a\-zA\-Z0\-9\_\\\-\\.\]\*
    :param str name: If set\, update the name for this provider  \=\> \[\\s\\w\\\{\\\}\\\$\\\-\\\(\\\)\\.\\\[\\\]\"\\\'\_\/\\\\\,\\\*\\\+\\\#\:\@\!\?\;\=\]\*
    :param str clientID: If set\, update the OAuth ClientID for this provider 
    :param str issuer: If set\, update the OpenID Issuer to require for this provider. Use blank string \"\" to reset to default. 
    :param str jwks: If set\, update the JWKS for this provider. Must be Base64\-encoding of a valid JWKS file. This will remove any stored certificate. 
    :param str certificate: If set\, update the X509 certificate for this provider. This will remove any stored jwks. 
    :param str providerURI: If set\, update the URI for this provider  \=\> \(\(https\?\|ftp\|gopher\|telnet\|file\)\:\(\(\/\)\|\(\\\\\)\)\+\[\\w\\d\:\\\#\@\%\/\;\$\(\)\~\_\?\\\\\+\-\=\\\\\\.\&\]\*\)
    :param str responseType: If set\, update the response type to request from provider 
    :param str authorizationPrompt: If set\, update the prompt to request from provider on authorization requests 
    :param dict claimsMapping: 
    :param list scope: If set\, change the scopes to request from this provider  \=\> Sanitize by regex \[\\x21\\x5D\-\\x7E\\x23\-\\x5B\]\+
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises InvalidArgumentsException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/authentication/v1/openid/provider/{idOrShortname}".format(idOrShortname=idOrShortname)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}
    # Only send shortName if the argument was provided, dont send null values
    if shortName is not None:
        body.update({"shortName": shortName})
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({"name": name})
    # Only send clientID if the argument was provided, dont send null values
    if clientID is not None:
        body.update({"clientID": clientID})
    # Only send issuer if the argument was provided, dont send null values
    if issuer is not None:
        body.update({"issuer": issuer})
    # Only send jwks if the argument was provided, dont send null values
    if jwks is not None:
        body.update({"jwks": jwks})
    # Only send certificate if the argument was provided, dont send null values
    if certificate is not None:
        body.update({"certificate": certificate})
    # Only send providerURI if the argument was provided, dont send null values
    if providerURI is not None:
        body.update({"providerURI": providerURI})
    # Only send responseType if the argument was provided, dont send null values
    if responseType is not None:
        body.update({"responseType": responseType})
    # Only send authorizationPrompt if the argument was provided, dont send null values
    if authorizationPrompt is not None:
        body.update({"authorizationPrompt": authorizationPrompt})
    # Only send claimsMapping if the argument was provided, dont send null values
    if claimsMapping is not None:
        body.update({"claimsMapping": claimsMapping})
    # Only send scope if the argument was provided, dont send null values
    if scope is not None:
        body.update({"scope": scope})

    query_parameters = {}
    log.debug("PUT %s (headers: %s, body: %s)" % (route, str(headers), str(body) or ""))

    response = session.put(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response

#: **DEPRECATED** : ``add_open_id_provider`` is an alias for ``add_openid_provider``. Exists
#: only for backward compatibility - **do not use** - use ``add_openid_provider`` instead.
add_open_id_provider = register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module,
    alias="add_open_id_provider"
)(deprecated_alias("add_open_id_provider")(add_openid_provider))
#: **DEPRECATED** : ``delete_open_id_provider`` is an alias for ``delete_openid_provider``. Exists
#: only for backward compatibility - **do not use** - use ``delete_openid_provider`` instead.
delete_open_id_provider = register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module,
    alias="delete_open_id_provider"
)(deprecated_alias("delete_open_id_provider")(delete_openid_provider))
#: **DEPRECATED** : ``get_open_id_provider`` is an alias for ``get_openid_provider``. Exists
#: only for backward compatibility - **do not use** - use ``get_openid_provider`` instead.
get_open_id_provider = register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module,
    alias="get_open_id_provider"
)(deprecated_alias("get_open_id_provider")(get_openid_provider))
#: **DEPRECATED** : ``list_open_id_providers`` is an alias for ``list_openid_providers``. Exists
#: only for backward compatibility - **do not use** - use ``list_openid_providers`` instead.
list_open_id_providers = register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module,
    alias="list_open_id_providers"
)(deprecated_alias("list_open_id_providers")(list_openid_providers))
#: **DEPRECATED** : ``open_id_authentication`` is an alias for ``openid_authentication``. Exists
#: only for backward compatibility - **do not use** - use ``openid_authentication`` instead.
open_id_authentication = register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module,
    alias="open_id_authentication"
)(deprecated_alias("open_id_authentication")(openid_authentication))
#: **DEPRECATED** : ``open_id_pre_authentication`` is an alias for ``openid_pre_authentication``. Exists
#: only for backward compatibility - **do not use** - use ``openid_pre_authentication`` instead.
open_id_pre_authentication = register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module,
    alias="open_id_pre_authentication"
)(deprecated_alias("open_id_pre_authentication")(openid_pre_authentication))
#: **DEPRECATED** : ``search_open_id_providers`` is an alias for ``search_openid_providers``. Exists
#: only for backward compatibility - **do not use** - use ``search_openid_providers`` instead.
search_open_id_providers = register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module,
    alias="search_open_id_providers"
)(deprecated_alias("search_open_id_providers")(search_openid_providers))
#: **DEPRECATED** : ``update_open_id_provider`` is an alias for ``update_openid_provider``. Exists
#: only for backward compatibility - **do not use** - use ``update_openid_provider`` instead.
update_open_id_provider = register_command(
    extending=("authentication", "v1", "openid", "authorize"),
    module=argus_cli_module,
    alias="update_open_id_provider"
)(deprecated_alias("update_open_id_provider")(update_openid_provider))