"""Autogenerated API"""
from argus_api import session
from requests import Response


def add_host_asset(
    ownerID: int = None,
    customerID: int = None,
    name: str = None,
    description: str = None,
    criticality: dict = None,
    properties: dict = None,
    type: str = None,
    source: str = None,
    operatingSystemCPE: str = None,
    ipAddresses: str = None,
    aliases: str = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Creates a new HostAsset. (PUBLIC)
    
    :param int ownerID: User who owns the asset. 
    :param int customerID: Customer the asset belongs to. 
    :param str name: Name of the asset.  \=\> \[\\s\\w\\\{\\\}\\\$\\\-\\\(\\\)\\.\\\[\\\]\"\\\'\_\/\\\\\,\\\*\\\+\\\#\:\@\!\?\;\=\]\*
    :param str description: Description of the asset.  \=\> \[\\s\\w\\\{\\\}\\\$\\\-\\\(\\\)\\.\\\[\\\]\"\\\'\_\/\\\\\,\\\*\\\+\\\#\:\@\!\?\;\=\]\*
    :param dict criticality: 
    :param dict properties: Custom user\-defined properties.  \=\> \[\\s\\w\\\{\\\}\\\$\\\-\\\(\\\)\\.\\\[\\\]\"\\\'\_\/\\\\\,\\\*\\\+\\\#\:\@\!\?\;\=\]\*
    :param str type: Defines if host is a client or a server. \(default SERVER\)
    :param str source: Source of the request. \(default USER\)
    :param str operatingSystemCPE: CPE of the host operating system. 
    :param list ipAddresses: IP address\(es\) of the host. 
    :param list aliases: Aliases \(domain names\) of the host. 
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/assets/v1/host".format()

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}
    # Only send ownerID if the argument was provided, dont send null values
    if ownerID is not None:
        body.update({"ownerID": ownerID})
    # Only send customerID if the argument was provided, dont send null values
    if customerID is not None:
        body.update({"customerID": customerID})
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({"name": name})
    # Only send description if the argument was provided, dont send null values
    if description is not None:
        body.update({"description": description})
    # Only send criticality if the argument was provided, dont send null values
    if criticality is not None:
        body.update({"criticality": criticality})
    # Only send properties if the argument was provided, dont send null values
    if properties is not None:
        body.update({"properties": properties})
    # Only send type if the argument was provided, dont send null values
    if type is not None:
        body.update({"type": type})
    # Only send source if the argument was provided, dont send null values
    if source is not None:
        body.update({"source": source})
    # Only send operatingSystemCPE if the argument was provided, dont send null values
    if operatingSystemCPE is not None:
        body.update({"operatingSystemCPE": operatingSystemCPE})
    # Only send ipAddresses if the argument was provided, dont send null values
    if ipAddresses is not None:
        body.update({"ipAddresses": ipAddresses})
    # Only send aliases if the argument was provided, dont send null values
    if aliases is not None:
        body.update({"aliases": aliases})

    query_parameters = {}

    response = session.post(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def bulk_update_host_asset(
    source: str = None,
    assetVulnerabilityAddRequests: dict = None,
    assetVulnerabilityUpdateRequests: dict = None,
    assetVulnerabilityResolveRequests: dict = None,
    assetVulnerabilityDeleteRequests: dict = None,
    hostApplicationAddRequests: dict = None,
    hostApplicationUpdateRequests: dict = None,
    hostApplicationDeleteRequests: dict = None,
    hostAssetAddRequests: dict = None,
    hostAssetUpdateRequests: dict = None,
    hostAssetDeleteRequests: dict = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Performs multiple updates to HostAssets in a single transaction. (PUBLIC)
    
    :param str source: Source of the request. \(default USER\)
    :param list assetVulnerabilityAddRequests: List of AssetVulnerabilityAddRequests. 
    :param list assetVulnerabilityUpdateRequests: List of AssetVulnerabilityUpdateRequests. 
    :param list assetVulnerabilityResolveRequests: List of AssetVulnerabilityResolveRequests. 
    :param list assetVulnerabilityDeleteRequests: List of AssetVulnerabilityDeleteRequests. 
    :param list hostApplicationAddRequests: List of HostApplicationAddRequests. 
    :param list hostApplicationUpdateRequests: List of HostApplicationUpdateRequests. 
    :param list hostApplicationDeleteRequests: List of HostApplicationDeleteRequests. 
    :param list hostAssetAddRequests: List of HostAssetAddRequests. Adding vulnerabilities\/applications to added hosts must be done in separate transaction. 
    :param list hostAssetUpdateRequests: List of HostAssetUpdateRequests. 
    :param list hostAssetDeleteRequests: List of HostAssetDeleteRequests. 
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/assets/v1/host".format()

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}
    # Only send source if the argument was provided, dont send null values
    if source is not None:
        body.update({"source": source})
    # Only send assetVulnerabilityAddRequests if the argument was provided, dont send null values
    if assetVulnerabilityAddRequests is not None:
        body.update({"assetVulnerabilityAddRequests": assetVulnerabilityAddRequests})
    # Only send assetVulnerabilityUpdateRequests if the argument was provided, dont send null values
    if assetVulnerabilityUpdateRequests is not None:
        body.update({"assetVulnerabilityUpdateRequests": assetVulnerabilityUpdateRequests})
    # Only send assetVulnerabilityResolveRequests if the argument was provided, dont send null values
    if assetVulnerabilityResolveRequests is not None:
        body.update({"assetVulnerabilityResolveRequests": assetVulnerabilityResolveRequests})
    # Only send assetVulnerabilityDeleteRequests if the argument was provided, dont send null values
    if assetVulnerabilityDeleteRequests is not None:
        body.update({"assetVulnerabilityDeleteRequests": assetVulnerabilityDeleteRequests})
    # Only send hostApplicationAddRequests if the argument was provided, dont send null values
    if hostApplicationAddRequests is not None:
        body.update({"hostApplicationAddRequests": hostApplicationAddRequests})
    # Only send hostApplicationUpdateRequests if the argument was provided, dont send null values
    if hostApplicationUpdateRequests is not None:
        body.update({"hostApplicationUpdateRequests": hostApplicationUpdateRequests})
    # Only send hostApplicationDeleteRequests if the argument was provided, dont send null values
    if hostApplicationDeleteRequests is not None:
        body.update({"hostApplicationDeleteRequests": hostApplicationDeleteRequests})
    # Only send hostAssetAddRequests if the argument was provided, dont send null values
    if hostAssetAddRequests is not None:
        body.update({"hostAssetAddRequests": hostAssetAddRequests})
    # Only send hostAssetUpdateRequests if the argument was provided, dont send null values
    if hostAssetUpdateRequests is not None:
        body.update({"hostAssetUpdateRequests": hostAssetUpdateRequests})
    # Only send hostAssetDeleteRequests if the argument was provided, dont send null values
    if hostAssetDeleteRequests is not None:
        body.update({"hostAssetDeleteRequests": hostAssetDeleteRequests})

    query_parameters = {}

    response = session.put(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def delete_host_asset(
    id: str,
    source: str = "USER",
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Marks a HostAsset as deleted. (PUBLIC)
    
    :param str id: HostAsset ID
    :param str source: Request source \(default USER\)
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/assets/v1/host/{id}".format(source=source,
        id=id)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}

    query_parameters = {}
    # Only send source if the argument was provided, dont send null values
    if source is not None:
        query_parameters.update({"source": source})
    

    response = session.delete(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def get_host_asset(
    id: str,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Returns a HostAsset identified by its ID. (PUBLIC)
    
    :param str id: HostAsset ID
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/assets/v1/host/{id}".format(id=id)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}

    query_parameters = {}

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def merge_host_asset(
    mergeIntoID: str,
    mergeFromID: str = None,
    ownerID: int = None,
    name: str = None,
    description: str = None,
    criticality: dict = None,
    type: str = None,
    mergeApplications: bool = None,
    mergeVulnerabilities: bool = None,
    operatingSystemCPE: str = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Merges multiple HostAssets into one. (INTERNAL)
    
    :param str mergeIntoID: ID of HostAsset to merge into. This HostAsset is kept and contains the merge result.
    :param list mergeFromID: Merge from hosts identified by their IDs. 
    :param int ownerID: Set owner of merged host. Keeps owner of merge into host if not provided. 
    :param str name: Set name of merged host. Keeps name of merge into host if not provided.  \=\> \[\\s\\w\\\{\\\}\\\$\\\-\\\(\\\)\\.\\\[\\\]\"\\\'\_\/\\\\\,\\\*\\\+\\\#\:\@\!\?\;\=\]\*
    :param str description: Set description of merged host. Keeps description of merge into host if not provided.  \=\> \[\\s\\w\\\{\\\}\\\$\\\-\\\(\\\)\\.\\\[\\\]\"\\\'\_\/\\\\\,\\\*\\\+\\\#\:\@\!\?\;\=\]\*
    :param dict criticality: 
    :param str type: Set type of merged host. Keeps type of merge into host if not provided. 
    :param bool mergeApplications: If set to true also merge applications \(defaults to false\). 
    :param bool mergeVulnerabilities: If set to true also merge vulnerabilities \(defaults to false\). 
    :param str operatingSystemCPE: Set operating system CPE of merged host. Keeps CPE of merge into host if not provided. 
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/assets/v1/host/{mergeIntoID}/merge".format(mergeIntoID=mergeIntoID)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}
    # Only send mergeFromID if the argument was provided, dont send null values
    if mergeFromID is not None:
        body.update({"mergeFromID": mergeFromID})
    # Only send ownerID if the argument was provided, dont send null values
    if ownerID is not None:
        body.update({"ownerID": ownerID})
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({"name": name})
    # Only send description if the argument was provided, dont send null values
    if description is not None:
        body.update({"description": description})
    # Only send criticality if the argument was provided, dont send null values
    if criticality is not None:
        body.update({"criticality": criticality})
    # Only send type if the argument was provided, dont send null values
    if type is not None:
        body.update({"type": type})
    # Only send mergeApplications if the argument was provided, dont send null values
    if mergeApplications is not None:
        body.update({"mergeApplications": mergeApplications})
    # Only send mergeVulnerabilities if the argument was provided, dont send null values
    if mergeVulnerabilities is not None:
        body.update({"mergeVulnerabilities": mergeVulnerabilities})
    # Only send operatingSystemCPE if the argument was provided, dont send null values
    if operatingSystemCPE is not None:
        body.update({"operatingSystemCPE": operatingSystemCPE})

    query_parameters = {}

    response = session.put(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def search_host_asset_statistics(
    name: str = None,
    startTimestamp: int = None,
    endTimestamp: int = None,
    keywords: str = None,
    keywordMatchStrategy: str = None,
    timeMatchStrategy: str = None,
    hostID: str = None,
    serviceID: str = None,
    businessProcessID: str = None,
    ipRange: str = None,
    applicationPort: int = None,
    applicationProtocol: str = None,
    cpe: str = None,
    hostCPE: str = None,
    applicationCPE: str = None,
    countTimeout: int = None,
    limit: int = None,
    offset: int = None,
    includeDeleted: bool = None,
    subCriteria: dict = None,
    exclude: bool = None,
    required: bool = None,
    customerID: int = None,
    ownerID: int = None,
    criticality: dict = None,
    minimumTotalCvss: float = None,
    maximumTotalCvss: float = None,
    vulnerabilityReference: str = None,
    vulnerabilityID: str = None,
    applicationRole: int = None,
    type: str = None,
    timeFieldStrategy: str = None,
    keywordFieldStrategy: str = None,
    includeVulnerabilityRawOutput: bool = None,
    includeVulnerabilityConclusion: bool = None,
    includeVulnerabilitySolution: bool = None,
    includeVulnerabilities: bool = None,
    includeApplications: bool = None,
    includeServices: bool = None,
    connectedToService: bool = None,
    groupBy: str = None,
    values: str = None,
    resolution: int = None,
    cutoff: int = None,
    cutoffValue: str = None,
    includeOthers: bool = None,
    sortBy: str = None,
    includeFlags: str = None,
    excludeFlags: str = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Fetch host asset statistics base on criteria (PUBLIC)
    
    :param list name: Restrict search to specific asset name
    :param int startTimestamp: Restrict search to a time frame based on the set TimeFieldStrategy \(start timestamp\).
    :param int endTimestamp: Restrict search to a time frame based on the set TimeFieldStrategy \(end timestamp\).
    :param list keywords: Search for keywords.
    :param str keywordMatchStrategy: Defines the MatchStrategy for keywords \(default match all keywords\).
    :param str timeMatchStrategy: Defines how strict to match against different timestamps \(all\/any\) using start and end timestamp \(default any\)
    :param list hostID: Restrict search to specific host UUIDs.
    :param list serviceID: Restrict search to specific service UUIDs.
    :param list businessProcessID: Restrict search to specific business process UUIDs.
    :param str ipRange: Restrict search to entities related to these IP\-addresses \(may specify single IPs\, IP networks or IP ranges.
    :param list applicationPort: Restrict to applications listening on specific ports.
    :param list applicationProtocol: Restrict to applications by transport protocol name.
    :param str cpe: Restrict to applications or hosts by CPE.
    :param str hostCPE: Restrict to hosts by CPE.
    :param str applicationCPE: Restrict to applications by CPE.
    :param int countTimeout: Max wait in seconds for count query\, if max wait passed and count query still executing the query\; or \-1 is set to indicate do not count\, then the query will be cancelled and return count \-1. Default value 0 means execute as long as service allows\, \-1 means do not count\, max value is 30 \(seconds\).
    :param int limit: Set this value to set max number of results. By default\, no restriction on result set size. 
    :param int offset: Set this value to skip the first \(offset\) objects. By default\, return result from first object. 
    :param bool includeDeleted: Set to true to include deleted objects. By default\, exclude deleted objects. 
    :param list subCriteria: Set additional criterias which are applied using a logical OR. 
    :param bool exclude: Only relevant for subcriteria. If set to true\, objects matching this subcriteria object will be excluded. 
    :param bool required: Only relevant for subcriteria. If set to true\, objects matching this subcriteria are required \(AND\-ed together with parent criteria\). 
    :param list customerID: Restrict search to data belonging to specified customers. 
    :param list ownerID: Restrict search to specific ownerIDs 
    :param list criticality: Restrict search to a range of criticality levels \(add multiple CriticalitySearch objects to specify OR criteria\). 
    :param float minimumTotalCvss: Restrict search to a minimum total CVSS score. 
    :param float maximumTotalCvss: Restrict search to a maximum total CVSS score. 
    :param list vulnerabilityReference: Restrict to vulnerabilities identified by vulnerability reference. 
    :param list vulnerabilityID: Restrict to vulnerabilities identified by vulnerability ID. 
    :param list applicationRole: Restrict to applications with specific roles \(list of role IDs\). 
    :param str type: Restrict search to a specific type of host \(client or server\). 
    :param list timeFieldStrategy: Defines which timestamps will be included in the search \(default lastUpdatedTimestamp on host\). 
    :param list keywordFieldStrategy: Defines which fields will be searched by keywords \(default all supported fields\). 
    :param bool includeVulnerabilityRawOutput: Include vulnerability rawOutput in result \(default false\). 
    :param bool includeVulnerabilityConclusion: Include vulnerability conclusion in result \(default false\). 
    :param bool includeVulnerabilitySolution: Include vulnerability solution in result \(default false\). 
    :param bool includeVulnerabilities: Include host vulnerabilities in result \(default false\). 
    :param bool includeApplications: Include host applications in result \(default false\). 
    :param bool includeServices: Include related services in result \(default false\). 
    :param bool connectedToService: If true\, only return hosts connected to service\(s\). If false\, return hosts not connected to any service. If not set\, do not filter. 
    :param list groupBy: Specify which fields will be grouped by in stats 
    :param list values: Specify which values will be included in stats \(default hosts\) 
    :param int resolution: Stats resolution period in milliseconds\, if is 0 means to generate non\-timeline statistics 
    :param int cutoff: Reduce stats keys to the cutoff amount of keys that has largest hit count\, default 0 means no reduce 
    :param str cutoffValue: Specify which value to be cutoff on \(default hosts\) 
    :param bool includeOthers: If reduce \(cutoff\>0\)\, true means remaining keys \(other than cutoff keys\) are collected into an \"other\" key\, default false 
    :param list sortBy: List of properties to sort by \(prefix with \"\-\" to sort descending\). 
    :param list includeFlags: Only include objects which have includeFlags set. 
    :param list excludeFlags: Exclude objects which have excludeFlags set. 
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationFailedException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/assets/v1/host/statistics".format()

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({"name": name})
    # Only send startTimestamp if the argument was provided, dont send null values
    if startTimestamp is not None:
        body.update({"startTimestamp": startTimestamp})
    # Only send endTimestamp if the argument was provided, dont send null values
    if endTimestamp is not None:
        body.update({"endTimestamp": endTimestamp})
    # Only send keywords if the argument was provided, dont send null values
    if keywords is not None:
        body.update({"keywords": keywords})
    # Only send keywordMatchStrategy if the argument was provided, dont send null values
    if keywordMatchStrategy is not None:
        body.update({"keywordMatchStrategy": keywordMatchStrategy})
    # Only send timeMatchStrategy if the argument was provided, dont send null values
    if timeMatchStrategy is not None:
        body.update({"timeMatchStrategy": timeMatchStrategy})
    # Only send hostID if the argument was provided, dont send null values
    if hostID is not None:
        body.update({"hostID": hostID})
    # Only send serviceID if the argument was provided, dont send null values
    if serviceID is not None:
        body.update({"serviceID": serviceID})
    # Only send businessProcessID if the argument was provided, dont send null values
    if businessProcessID is not None:
        body.update({"businessProcessID": businessProcessID})
    # Only send ipRange if the argument was provided, dont send null values
    if ipRange is not None:
        body.update({"ipRange": ipRange})
    # Only send applicationPort if the argument was provided, dont send null values
    if applicationPort is not None:
        body.update({"applicationPort": applicationPort})
    # Only send applicationProtocol if the argument was provided, dont send null values
    if applicationProtocol is not None:
        body.update({"applicationProtocol": applicationProtocol})
    # Only send cpe if the argument was provided, dont send null values
    if cpe is not None:
        body.update({"cpe": cpe})
    # Only send hostCPE if the argument was provided, dont send null values
    if hostCPE is not None:
        body.update({"hostCPE": hostCPE})
    # Only send applicationCPE if the argument was provided, dont send null values
    if applicationCPE is not None:
        body.update({"applicationCPE": applicationCPE})
    # Only send countTimeout if the argument was provided, dont send null values
    if countTimeout is not None:
        body.update({"countTimeout": countTimeout})
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        body.update({"limit": limit})
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        body.update({"offset": offset})
    # Only send includeDeleted if the argument was provided, dont send null values
    if includeDeleted is not None:
        body.update({"includeDeleted": includeDeleted})
    # Only send subCriteria if the argument was provided, dont send null values
    if subCriteria is not None:
        body.update({"subCriteria": subCriteria})
    # Only send exclude if the argument was provided, dont send null values
    if exclude is not None:
        body.update({"exclude": exclude})
    # Only send required if the argument was provided, dont send null values
    if required is not None:
        body.update({"required": required})
    # Only send customerID if the argument was provided, dont send null values
    if customerID is not None:
        body.update({"customerID": customerID})
    # Only send ownerID if the argument was provided, dont send null values
    if ownerID is not None:
        body.update({"ownerID": ownerID})
    # Only send criticality if the argument was provided, dont send null values
    if criticality is not None:
        body.update({"criticality": criticality})
    # Only send minimumTotalCvss if the argument was provided, dont send null values
    if minimumTotalCvss is not None:
        body.update({"minimumTotalCvss": minimumTotalCvss})
    # Only send maximumTotalCvss if the argument was provided, dont send null values
    if maximumTotalCvss is not None:
        body.update({"maximumTotalCvss": maximumTotalCvss})
    # Only send vulnerabilityReference if the argument was provided, dont send null values
    if vulnerabilityReference is not None:
        body.update({"vulnerabilityReference": vulnerabilityReference})
    # Only send vulnerabilityID if the argument was provided, dont send null values
    if vulnerabilityID is not None:
        body.update({"vulnerabilityID": vulnerabilityID})
    # Only send applicationRole if the argument was provided, dont send null values
    if applicationRole is not None:
        body.update({"applicationRole": applicationRole})
    # Only send type if the argument was provided, dont send null values
    if type is not None:
        body.update({"type": type})
    # Only send timeFieldStrategy if the argument was provided, dont send null values
    if timeFieldStrategy is not None:
        body.update({"timeFieldStrategy": timeFieldStrategy})
    # Only send keywordFieldStrategy if the argument was provided, dont send null values
    if keywordFieldStrategy is not None:
        body.update({"keywordFieldStrategy": keywordFieldStrategy})
    # Only send includeVulnerabilityRawOutput if the argument was provided, dont send null values
    if includeVulnerabilityRawOutput is not None:
        body.update({"includeVulnerabilityRawOutput": includeVulnerabilityRawOutput})
    # Only send includeVulnerabilityConclusion if the argument was provided, dont send null values
    if includeVulnerabilityConclusion is not None:
        body.update({"includeVulnerabilityConclusion": includeVulnerabilityConclusion})
    # Only send includeVulnerabilitySolution if the argument was provided, dont send null values
    if includeVulnerabilitySolution is not None:
        body.update({"includeVulnerabilitySolution": includeVulnerabilitySolution})
    # Only send includeVulnerabilities if the argument was provided, dont send null values
    if includeVulnerabilities is not None:
        body.update({"includeVulnerabilities": includeVulnerabilities})
    # Only send includeApplications if the argument was provided, dont send null values
    if includeApplications is not None:
        body.update({"includeApplications": includeApplications})
    # Only send includeServices if the argument was provided, dont send null values
    if includeServices is not None:
        body.update({"includeServices": includeServices})
    # Only send connectedToService if the argument was provided, dont send null values
    if connectedToService is not None:
        body.update({"connectedToService": connectedToService})
    # Only send groupBy if the argument was provided, dont send null values
    if groupBy is not None:
        body.update({"groupBy": groupBy})
    # Only send values if the argument was provided, dont send null values
    if values is not None:
        body.update({"values": values})
    # Only send resolution if the argument was provided, dont send null values
    if resolution is not None:
        body.update({"resolution": resolution})
    # Only send cutoff if the argument was provided, dont send null values
    if cutoff is not None:
        body.update({"cutoff": cutoff})
    # Only send cutoffValue if the argument was provided, dont send null values
    if cutoffValue is not None:
        body.update({"cutoffValue": cutoffValue})
    # Only send includeOthers if the argument was provided, dont send null values
    if includeOthers is not None:
        body.update({"includeOthers": includeOthers})
    # Only send sortBy if the argument was provided, dont send null values
    if sortBy is not None:
        body.update({"sortBy": sortBy})
    # Only send includeFlags if the argument was provided, dont send null values
    if includeFlags is not None:
        body.update({"includeFlags": includeFlags})
    # Only send excludeFlags if the argument was provided, dont send null values
    if excludeFlags is not None:
        body.update({"excludeFlags": excludeFlags})

    query_parameters = {}

    response = session.post(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def search_host_assets(
    name: str = None,
    startTimestamp: int = None,
    endTimestamp: int = None,
    keywords: str = None,
    keywordMatchStrategy: str = None,
    timeMatchStrategy: str = None,
    hostID: str = None,
    serviceID: str = None,
    businessProcessID: str = None,
    ipRange: str = None,
    applicationPort: int = None,
    applicationProtocol: str = None,
    cpe: str = None,
    hostCPE: str = None,
    applicationCPE: str = None,
    countTimeout: int = None,
    limit: int = None,
    offset: int = None,
    includeDeleted: bool = None,
    subCriteria: dict = None,
    exclude: bool = None,
    required: bool = None,
    customerID: int = None,
    ownerID: int = None,
    criticality: dict = None,
    minimumTotalCvss: float = None,
    maximumTotalCvss: float = None,
    vulnerabilityReference: str = None,
    vulnerabilityID: str = None,
    applicationRole: int = None,
    type: str = None,
    timeFieldStrategy: str = None,
    keywordFieldStrategy: str = None,
    includeVulnerabilityRawOutput: bool = None,
    includeVulnerabilityConclusion: bool = None,
    includeVulnerabilitySolution: bool = None,
    includeVulnerabilities: bool = None,
    includeApplications: bool = None,
    includeServices: bool = None,
    connectedToService: bool = None,
    sortBy: str = None,
    includeFlags: str = None,
    excludeFlags: str = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Returns a set of HostAssets defined by a HostAssetSearchCriteria. (PUBLIC)
    
    :param list name: Restrict search to specific asset name
    :param int startTimestamp: Restrict search to a time frame based on the set TimeFieldStrategy \(start timestamp\).
    :param int endTimestamp: Restrict search to a time frame based on the set TimeFieldStrategy \(end timestamp\).
    :param list keywords: Search for keywords.
    :param str keywordMatchStrategy: Defines the MatchStrategy for keywords \(default match all keywords\).
    :param str timeMatchStrategy: Defines how strict to match against different timestamps \(all\/any\) using start and end timestamp \(default any\)
    :param list hostID: Restrict search to specific host UUIDs.
    :param list serviceID: Restrict search to specific service UUIDs.
    :param list businessProcessID: Restrict search to specific business process UUIDs.
    :param str ipRange: Restrict search to entities related to these IP\-addresses \(may specify single IPs\, IP networks or IP ranges.
    :param list applicationPort: Restrict to applications listening on specific ports.
    :param list applicationProtocol: Restrict to applications by transport protocol name.
    :param str cpe: Restrict to applications or hosts by CPE.
    :param str hostCPE: Restrict to hosts by CPE.
    :param str applicationCPE: Restrict to applications by CPE.
    :param int countTimeout: Max wait in seconds for count query\, if max wait passed and count query still executing the query\; or \-1 is set to indicate do not count\, then the query will be cancelled and return count \-1. Default value 0 means execute as long as service allows\, \-1 means do not count\, max value is 30 \(seconds\).
    :param int limit: Set this value to set max number of results. By default\, no restriction on result set size. 
    :param int offset: Set this value to skip the first \(offset\) objects. By default\, return result from first object. 
    :param bool includeDeleted: Set to true to include deleted objects. By default\, exclude deleted objects. 
    :param list subCriteria: Set additional criterias which are applied using a logical OR. 
    :param bool exclude: Only relevant for subcriteria. If set to true\, objects matching this subcriteria object will be excluded. 
    :param bool required: Only relevant for subcriteria. If set to true\, objects matching this subcriteria are required \(AND\-ed together with parent criteria\). 
    :param list customerID: Restrict search to data belonging to specified customers. 
    :param list ownerID: Restrict search to specific ownerIDs 
    :param list criticality: Restrict search to a range of criticality levels \(add multiple CriticalitySearch objects to specify OR criteria\). 
    :param float minimumTotalCvss: Restrict search to a minimum total CVSS score. 
    :param float maximumTotalCvss: Restrict search to a maximum total CVSS score. 
    :param list vulnerabilityReference: Restrict to vulnerabilities identified by vulnerability reference. 
    :param list vulnerabilityID: Restrict to vulnerabilities identified by vulnerability ID. 
    :param list applicationRole: Restrict to applications with specific roles \(list of role IDs\). 
    :param str type: Restrict search to a specific type of host \(client or server\). 
    :param list timeFieldStrategy: Defines which timestamps will be included in the search \(default lastUpdatedTimestamp on host\). 
    :param list keywordFieldStrategy: Defines which fields will be searched by keywords \(default all supported fields\). 
    :param bool includeVulnerabilityRawOutput: Include vulnerability rawOutput in result \(default false\). 
    :param bool includeVulnerabilityConclusion: Include vulnerability conclusion in result \(default false\). 
    :param bool includeVulnerabilitySolution: Include vulnerability solution in result \(default false\). 
    :param bool includeVulnerabilities: Include host vulnerabilities in result \(default false\). 
    :param bool includeApplications: Include host applications in result \(default false\). 
    :param bool includeServices: Include related services in result \(default false\). 
    :param bool connectedToService: If true\, only return hosts connected to service\(s\). If false\, return hosts not connected to any service. If not set\, do not filter. 
    :param list sortBy: List of properties to sort by \(prefix with \"\-\" to sort descending\). 
    :param list includeFlags: Only include objects which have includeFlags set. 
    :param list excludeFlags: Exclude objects which have excludeFlags set. 
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes
    
    :returns: ``requests.Response`` object or dictionary translated from JSON
    """

    route = "/assets/v1/host/search".format()

    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': None
    }
    if json:
        headers['content'] = 'application/json'

    body = body or {}
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({"name": name})
    # Only send startTimestamp if the argument was provided, dont send null values
    if startTimestamp is not None:
        body.update({"startTimestamp": startTimestamp})
    # Only send endTimestamp if the argument was provided, dont send null values
    if endTimestamp is not None:
        body.update({"endTimestamp": endTimestamp})
    # Only send keywords if the argument was provided, dont send null values
    if keywords is not None:
        body.update({"keywords": keywords})
    # Only send keywordMatchStrategy if the argument was provided, dont send null values
    if keywordMatchStrategy is not None:
        body.update({"keywordMatchStrategy": keywordMatchStrategy})
    # Only send timeMatchStrategy if the argument was provided, dont send null values
    if timeMatchStrategy is not None:
        body.update({"timeMatchStrategy": timeMatchStrategy})
    # Only send hostID if the argument was provided, dont send null values
    if hostID is not None:
        body.update({"hostID": hostID})
    # Only send serviceID if the argument was provided, dont send null values
    if serviceID is not None:
        body.update({"serviceID": serviceID})
    # Only send businessProcessID if the argument was provided, dont send null values
    if businessProcessID is not None:
        body.update({"businessProcessID": businessProcessID})
    # Only send ipRange if the argument was provided, dont send null values
    if ipRange is not None:
        body.update({"ipRange": ipRange})
    # Only send applicationPort if the argument was provided, dont send null values
    if applicationPort is not None:
        body.update({"applicationPort": applicationPort})
    # Only send applicationProtocol if the argument was provided, dont send null values
    if applicationProtocol is not None:
        body.update({"applicationProtocol": applicationProtocol})
    # Only send cpe if the argument was provided, dont send null values
    if cpe is not None:
        body.update({"cpe": cpe})
    # Only send hostCPE if the argument was provided, dont send null values
    if hostCPE is not None:
        body.update({"hostCPE": hostCPE})
    # Only send applicationCPE if the argument was provided, dont send null values
    if applicationCPE is not None:
        body.update({"applicationCPE": applicationCPE})
    # Only send countTimeout if the argument was provided, dont send null values
    if countTimeout is not None:
        body.update({"countTimeout": countTimeout})
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        body.update({"limit": limit})
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        body.update({"offset": offset})
    # Only send includeDeleted if the argument was provided, dont send null values
    if includeDeleted is not None:
        body.update({"includeDeleted": includeDeleted})
    # Only send subCriteria if the argument was provided, dont send null values
    if subCriteria is not None:
        body.update({"subCriteria": subCriteria})
    # Only send exclude if the argument was provided, dont send null values
    if exclude is not None:
        body.update({"exclude": exclude})
    # Only send required if the argument was provided, dont send null values
    if required is not None:
        body.update({"required": required})
    # Only send customerID if the argument was provided, dont send null values
    if customerID is not None:
        body.update({"customerID": customerID})
    # Only send ownerID if the argument was provided, dont send null values
    if ownerID is not None:
        body.update({"ownerID": ownerID})
    # Only send criticality if the argument was provided, dont send null values
    if criticality is not None:
        body.update({"criticality": criticality})
    # Only send minimumTotalCvss if the argument was provided, dont send null values
    if minimumTotalCvss is not None:
        body.update({"minimumTotalCvss": minimumTotalCvss})
    # Only send maximumTotalCvss if the argument was provided, dont send null values
    if maximumTotalCvss is not None:
        body.update({"maximumTotalCvss": maximumTotalCvss})
    # Only send vulnerabilityReference if the argument was provided, dont send null values
    if vulnerabilityReference is not None:
        body.update({"vulnerabilityReference": vulnerabilityReference})
    # Only send vulnerabilityID if the argument was provided, dont send null values
    if vulnerabilityID is not None:
        body.update({"vulnerabilityID": vulnerabilityID})
    # Only send applicationRole if the argument was provided, dont send null values
    if applicationRole is not None:
        body.update({"applicationRole": applicationRole})
    # Only send type if the argument was provided, dont send null values
    if type is not None:
        body.update({"type": type})
    # Only send timeFieldStrategy if the argument was provided, dont send null values
    if timeFieldStrategy is not None:
        body.update({"timeFieldStrategy": timeFieldStrategy})
    # Only send keywordFieldStrategy if the argument was provided, dont send null values
    if keywordFieldStrategy is not None:
        body.update({"keywordFieldStrategy": keywordFieldStrategy})
    # Only send includeVulnerabilityRawOutput if the argument was provided, dont send null values
    if includeVulnerabilityRawOutput is not None:
        body.update({"includeVulnerabilityRawOutput": includeVulnerabilityRawOutput})
    # Only send includeVulnerabilityConclusion if the argument was provided, dont send null values
    if includeVulnerabilityConclusion is not None:
        body.update({"includeVulnerabilityConclusion": includeVulnerabilityConclusion})
    # Only send includeVulnerabilitySolution if the argument was provided, dont send null values
    if includeVulnerabilitySolution is not None:
        body.update({"includeVulnerabilitySolution": includeVulnerabilitySolution})
    # Only send includeVulnerabilities if the argument was provided, dont send null values
    if includeVulnerabilities is not None:
        body.update({"includeVulnerabilities": includeVulnerabilities})
    # Only send includeApplications if the argument was provided, dont send null values
    if includeApplications is not None:
        body.update({"includeApplications": includeApplications})
    # Only send includeServices if the argument was provided, dont send null values
    if includeServices is not None:
        body.update({"includeServices": includeServices})
    # Only send connectedToService if the argument was provided, dont send null values
    if connectedToService is not None:
        body.update({"connectedToService": connectedToService})
    # Only send sortBy if the argument was provided, dont send null values
    if sortBy is not None:
        body.update({"sortBy": sortBy})
    # Only send includeFlags if the argument was provided, dont send null values
    if includeFlags is not None:
        body.update({"includeFlags": includeFlags})
    # Only send excludeFlags if the argument was provided, dont send null values
    if excludeFlags is not None:
        body.update({"excludeFlags": excludeFlags})

    query_parameters = {}

    response = session.post(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def search_host_assets_simplified(
    keywords: str = None,
    keywordField: str = None,
    name: str = None,
    hostID: str = None,
    serviceID: str = None,
    businessProcessID: str = None,
    customerID: int = None,
    ip: str = None,
    port: int = None,
    protocol: str = None,
    cpe: str = None,
    vulnID: str = None,
    vulnRef: str = None,
    includeFlag: str = None,
    excludeFlag: str = None,
    sortBy: str = None,
    countTimeout: int = None,
    limit: int = 25,
    keywordMatch: str = "all",
    offset: int = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Returns as set of HostAssets defined by query parameters. (PUBLIC)
    
    :param list keywords: Search by keywords
    :param list keywordField: Set field strategy for keyword search
    :param list name: Search by name
    :param list hostID: Search by HostAsset ID
    :param list serviceID: Search by ServiceAsset ID
    :param list businessProcessID: Search by BusinessProcess ID
    :param list customerID: Search by customer ID
    :param list ip: Search by IP range
    :param list port: Search by application port
    :param list protocol: Search by application protocol
    :param list cpe: Search by CPE
    :param list vulnID: Search by vulnerability ID
    :param list vulnRef: Search by vulnerability reference
    :param list includeFlag: Include certain HostAssets in the search result based on set flags
    :param list excludeFlag: Exclude certain HostAssets from the search result based on set flags
    :param list sortBy: Sort search result
    :param int countTimeout: Max wait in seconds for count query. Value 0 means execute as long as service allows\, \-1 means do not count\, max value is 30 \(seconds\).
    :param int limit: Maximum number of returned results
    :param str keywordMatch: Set match strategy for keyword search
    :param int offset: Skip a number of results
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes
    
    :returns: ``requests.Response`` object or dictionary translated from JSON
    """

    route = "/assets/v1/host".format(limit=limit,
        keywordMatch=keywordMatch,
        offset=offset,
        keywords=keywords,
        keywordField=keywordField,
        name=name,
        hostID=hostID,
        serviceID=serviceID,
        businessProcessID=businessProcessID,
        customerID=customerID,
        ip=ip,
        port=port,
        protocol=protocol,
        cpe=cpe,
        vulnID=vulnID,
        vulnRef=vulnRef,
        includeFlag=includeFlag,
        excludeFlag=excludeFlag,
        sortBy=sortBy,
        countTimeout=countTimeout)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': None
    }
    if json:
        headers['content'] = 'application/json'

    body = body or {}

    query_parameters = {}
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        query_parameters.update({"limit": limit})
    
    # Only send keywordMatch if the argument was provided, dont send null values
    if keywordMatch is not None:
        query_parameters.update({"keywordMatch": keywordMatch})
    
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        query_parameters.update({"offset": offset})
    
    # Only send keywords if the argument was provided, dont send null values
    if keywords is not None:
        query_parameters.update({"keywords": keywords})
    
    # Only send keywordField if the argument was provided, dont send null values
    if keywordField is not None:
        query_parameters.update({"keywordField": keywordField})
    
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        query_parameters.update({"name": name})
    
    # Only send hostID if the argument was provided, dont send null values
    if hostID is not None:
        query_parameters.update({"hostID": hostID})
    
    # Only send serviceID if the argument was provided, dont send null values
    if serviceID is not None:
        query_parameters.update({"serviceID": serviceID})
    
    # Only send businessProcessID if the argument was provided, dont send null values
    if businessProcessID is not None:
        query_parameters.update({"businessProcessID": businessProcessID})
    
    # Only send customerID if the argument was provided, dont send null values
    if customerID is not None:
        query_parameters.update({"customerID": customerID})
    
    # Only send ip if the argument was provided, dont send null values
    if ip is not None:
        query_parameters.update({"ip": ip})
    
    # Only send port if the argument was provided, dont send null values
    if port is not None:
        query_parameters.update({"port": port})
    
    # Only send protocol if the argument was provided, dont send null values
    if protocol is not None:
        query_parameters.update({"protocol": protocol})
    
    # Only send cpe if the argument was provided, dont send null values
    if cpe is not None:
        query_parameters.update({"cpe": cpe})
    
    # Only send vulnID if the argument was provided, dont send null values
    if vulnID is not None:
        query_parameters.update({"vulnID": vulnID})
    
    # Only send vulnRef if the argument was provided, dont send null values
    if vulnRef is not None:
        query_parameters.update({"vulnRef": vulnRef})
    
    # Only send includeFlag if the argument was provided, dont send null values
    if includeFlag is not None:
        query_parameters.update({"includeFlag": includeFlag})
    
    # Only send excludeFlag if the argument was provided, dont send null values
    if excludeFlag is not None:
        query_parameters.update({"excludeFlag": excludeFlag})
    
    # Only send sortBy if the argument was provided, dont send null values
    if sortBy is not None:
        query_parameters.update({"sortBy": sortBy})
    
    # Only send countTimeout if the argument was provided, dont send null values
    if countTimeout is not None:
        query_parameters.update({"countTimeout": countTimeout})
    

    response = session.get(
        route,
        params=query_parameters or None,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response


def update_host_asset(
    id: str,
    ownerID: int = None,
    name: str = None,
    description: str = None,
    criticality: dict = None,
    addProperties: dict = None,
    deleteProperties: str = None,
    type: str = None,
    source: str = None,
    operatingSystemCPE: str = None,
    addIpAddresses: str = None,
    deleteIpAddresses: str = None,
    addAliases: str = None,
    deleteAliases: str = None,
    json: bool = True,
    verify: bool = None,
    proxies: dict = None,
    apiKey: str = None,
    authentication: dict = {},
    server_url: str = None,
    body: dict = None,
) -> dict:
    """Updates an existing HostAsset. (PUBLIC)
    
    :param str id: HostAsset ID
    :param int ownerID: Change user who owns the asset. 
    :param str name: Change name of asset.  \=\> \[\\s\\w\\\{\\\}\\\$\\\-\\\(\\\)\\.\\\[\\\]\"\\\'\_\/\\\\\,\\\*\\\+\\\#\:\@\!\?\;\=\]\*
    :param str description: Change description of asset.  \=\> \[\\s\\w\\\{\\\}\\\$\\\-\\\(\\\)\\.\\\[\\\]\"\\\'\_\/\\\\\,\\\*\\\+\\\#\:\@\!\?\;\=\]\*
    :param dict criticality: 
    :param dict addProperties: Add custom properties \(updates a property if key already exists\).  \=\> \[\\s\\w\\\{\\\}\\\$\\\-\\\(\\\)\\.\\\[\\\]\"\\\'\_\/\\\\\,\\\*\\\+\\\#\:\@\!\?\;\=\]\*
    :param list deleteProperties: Delete custom properties by key. 
    :param str type: Change type of host \(client or server\). 
    :param str source: Source of the request. \(default USER\)
    :param str operatingSystemCPE: Change CPE of host. 
    :param list addIpAddresses: Add IP address\(es\) to host. 
    :param list deleteIpAddresses: Delete IP address\(es\) from host. 
    :param list addAliases: Add alias\(es\) \(domain names\) to host. 
    :param list deleteAliases: Delete alias\(es\) from host. 
    :param json: return the response's body as a ``dict`` parsed from json. ``True`` by
      default. If set to false, the raw ``requests.Response`` object will be returned.
    :param verify: path to a certificate bundle or boolean indicating whether SSL
      verification should be performed.
    :param apiKey: Argus API key.
    :param authentication: authentication override
    :param server_url: API base URL override
    :param body: body of the request. other parameters will override keys defined in the body.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412
    :raises ArgusException: on other status codes
    
    :returns: dictionary translated from JSON
    """

    route = "/assets/v1/host/{id}".format(id=id)

    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    body = body or {}
    # Only send ownerID if the argument was provided, dont send null values
    if ownerID is not None:
        body.update({"ownerID": ownerID})
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({"name": name})
    # Only send description if the argument was provided, dont send null values
    if description is not None:
        body.update({"description": description})
    # Only send criticality if the argument was provided, dont send null values
    if criticality is not None:
        body.update({"criticality": criticality})
    # Only send addProperties if the argument was provided, dont send null values
    if addProperties is not None:
        body.update({"addProperties": addProperties})
    # Only send deleteProperties if the argument was provided, dont send null values
    if deleteProperties is not None:
        body.update({"deleteProperties": deleteProperties})
    # Only send type if the argument was provided, dont send null values
    if type is not None:
        body.update({"type": type})
    # Only send source if the argument was provided, dont send null values
    if source is not None:
        body.update({"source": source})
    # Only send operatingSystemCPE if the argument was provided, dont send null values
    if operatingSystemCPE is not None:
        body.update({"operatingSystemCPE": operatingSystemCPE})
    # Only send addIpAddresses if the argument was provided, dont send null values
    if addIpAddresses is not None:
        body.update({"addIpAddresses": addIpAddresses})
    # Only send deleteIpAddresses if the argument was provided, dont send null values
    if deleteIpAddresses is not None:
        body.update({"deleteIpAddresses": deleteIpAddresses})
    # Only send addAliases if the argument was provided, dont send null values
    if addAliases is not None:
        body.update({"addAliases": addAliases})
    # Only send deleteAliases if the argument was provided, dont send null values
    if deleteAliases is not None:
        body.update({"deleteAliases": deleteAliases})

    query_parameters = {}

    response = session.put(
        route,
        params=query_parameters or None,
        json=body,
        verify=verify,
        apiKey=apiKey,
        authentication=authentication,
        server_url=server_url,
        headers=headers,
        proxies=proxies,
    )
    return response.json() if json else response
